/*	$Id: debug.h,v 1.11 2006-07-01 00:33:41 phil Exp $ */

/* BLURB gpl

                           Coda File System
                              Release 6

          Copyright (c) 1987-2005 Carnegie Mellon University
                  Additional copyrights listed below

This  code  is  distributed "AS IS" without warranty of any kind under
the terms of the GNU General Public Licence Version 2, as shown in the
file  LICENSE.  The  technical and financial  contributors to Coda are
listed in the file CREDITS.

*/


/*
 *  Coda NT kernel module ... debugging support.
 */

#define DBGENTRY

/* debugging support */

char *coda_f2s(CodaFid *);

void init_msg_list(void);
void clean_msg_list(void);

int get_debug_mesg (char *buffer, int buffersize);

// Messages that should print in the FREE build should call DbgPrint(...)
// directly.

#ifdef DBG
#define CDEBUG(x) coda_debug x
#else
#define CDEBUG(x)
#endif

void coda_debug (unsigned int mask, char *Format, ... );

extern ULONG DebugLevel;

/* debugging masks */
#define D_SUPER    0x0001   /* print results returned by Venus */ 
#define D_INODE    0x0002   /* print entry and exit into procedure */
#define D_FILE     0x0004   
#define D_CACHE    0x0008   /* cache debugging */
#define D_MALLOC   0x0010   /* print malloc, de-alloc information */
#define D_CNODE    0x0020
#define D_UPCALL   0x0040   /* up and downcall debugging */
#define D_IPC      0x0080   /* Actual comm between venus/fsd */
#define D_PIOCTL   0x0100
#define D_UID      0x0200   /* Things pertaining to the SID/UID */
#define D_WRAPPER  0x0400   /* track the wrappers calls to us */
#define D_LOOKATTR 0x0800   /* Wrapper lookup, getattr, release */ 
#define D_DOWNCALL 0x1000   /* Downcalls from venus */
#define D_MISC     0x2000   /* Other unclassified calls. */
#define D_DETAILS  0x4000   /* Used with others to allow some extra
			       filtering of what to show.  see coda.c */
#define D_FHAND	   0x8000   /* File handle debugging. */

#define D_GETMSG   0x400000   /* Save the message for a user to get via
			       a fsctl. */
#define D_PRCACHE  0x800000 /* Print the inode cache */

// Debugging defines.

#ifdef DBGENTRY
#define ENTRY(x,y)  CDEBUG((x,"%s: Starting call.\n",y))
#define EXIT(x,y)   CDEBUG((x,"%s: Returning at end.\n",y))
#else
#define ENTRY(x,y)
#define EXIT(x,y)
#endif

