// utility to dynamically set debug level in NT coda kernel module
//
//  ntdebug <level names>
//
//  $Id: ntdebug.c,v 1.6 2005/09/09 23:10:36 phil Exp $

/* BLURB gpl

                           Coda File System
                              Release 6

          Copyright (c) 2005 Carnegie Mellon University
                  Additional copyrights listed below

This  code  is  distributed "AS IS" without warranty of any kind under
the terms of the GNU General Public Licence Version 2, as shown in the
file  LICENSE.  The  technical and financial  contributors to Coda are
listed in the file CREDITS.

*/

#include <windows.h>
#include <stdio.h>
#include <winioctl.h>
#include <string.h>
#include <ctype.h>
#include <wchar.h>
#include "coda.h"
#include "codafsctl.h"
#include "debug.h"

struct debugnames {
    char *name;
    int  val;
};

struct debugnames dnames[] = {
    {"none", 0},
    {"super",    D_SUPER},
    {"inode",    D_INODE},
    {"file",     D_FILE},
    {"cache",    D_CACHE},
    {"malloc",   D_MALLOC},
    {"cnode",    D_CNODE},
    {"upcall",   D_UPCALL},
    {"ipc",      D_IPC},
    {"pioctl",   D_PIOCTL},
    {"uid",      D_UID},
    {"wrapper",  D_WRAPPER},
    {"lookattr", D_LOOKATTR},
    {"downcall", D_DOWNCALL},
    {"misc",     D_MISC},
    {"details",  D_DETAILS},
    {"fhand",    D_FHAND},
    {"getmsg",   D_GETMSG},
    {"prcache",  D_PRCACHE},
    {"all",      0xFFFFF},
    {0, 0}};


void myperror(char *str)
{
    DWORD err = GetLastError();
    char buf[1000];
    
    FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM, NULL, err, 0, buf, sizeof(buf),
		   NULL);
    printf ("%s: %s\n", str, buf);
    exit (-1);
}

void usage (char *progname)
{
    int ix;

    printf ("usage: %s none\n", progname);
    printf ("       %s debug-level-names\n\n", progname);
    printf ("debug level names are:\n");
    for (ix = 0; dnames[ix].name; ix++)
	printf ("     %s\n", dnames[ix].name);
    exit(1);
}

#define BUFSIZE 4096
int main (int argc, char *argv[])
{
    HANDLE h;
    DWORD DebugLevel = 0;
    LONG nBytesReturned;
    CHAR outBuf[BUFSIZE];
    DWORD d;
    int n;
    int ctlcode = CODA_FSCTL_SETDBGLEVEL;

    int i;
    
    
    if (argc > 1) {
	i = 1;
	while (i < argc) {
	    n = 0;
	    while (dnames[n].name) {
		if (strcmp(argv[i], dnames[n].name) == 0) {
		    DebugLevel |= dnames[n].val;
		    break;
		}
		n++;
	    }
	    if (!dnames[n].name)
		usage(argv[0]);
	    i++;
	}
    } else {
	usage (argv[0]);
    }

    printf ("Starting with DefineDosDevice ...\n");

    d = DefineDosDevice(DDD_RAW_TARGET_PATH, "codadev", "\\Device\\codadev");
    if ( d == 0 ) 
	myperror("DDD failed");
    h = CreateFile ("\\\\.\\codadev", GENERIC_READ | GENERIC_WRITE,
		    FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
		    OPEN_EXISTING, 0, NULL);
    if (h == INVALID_HANDLE_VALUE) {
	myperror ("CreateFile failed");
    }
    
    
    if ((d = DeviceIoControl(h, ctlcode, &DebugLevel, sizeof(DebugLevel),
			     NULL, 0, &nBytesReturned, NULL))) {
	printf ("Debug level set to 0x%x\n", DebugLevel);
    } else {
	myperror ("request failed\n");
	return 1;
    }
    
    return 0;
}

